import pylab
import numpy
from scipy.optimize import curve_fit

def fit_function(x, omega):
    """ Definition of the fit function.
    """
    return pylab.sin(omega*x)

# Load the data from the file.
t, s, ds = pylab.loadtxt('data/fit_initial_val.txt', unpack = True)

# Do the fit.
initial_values = (1.2,)
pars, covm = curve_fit(fit_function, t, s, initial_values, ds)
omega0 = pars[0]

# Plot the data points and the fit function.
pylab.rc('font', size = 18)
pylab.title('Law of motion fit', y = 1.02)
pylab.xlabel('t [s]')
pylab.ylabel('s [m]', labelpad = 5)
pylab.xlim(0, 20)
pylab.ylim(-1.5, 1.5)
pylab.grid(color = 'gray')
x = numpy.linspace(0, 20, 200)
y = fit_function(x, omega0)
pylab.errorbar(t, s, ds, linestyle = '', color = 'black', marker = 'o')
pylab.plot(x, y, color = 'black')

# Save the plot to a pdf file for later use (maybe in a writeup?).
pylab.savefig('fit_initial_val_fitplot.pdf')

# Finally: show the plot on the screen.
pylab.show()
