# This is a script to fit a pendulum position vs. time plot
# to extract frequency and dumping parameter
# matplotlib is used to plot and scipy for fitting
# 

# Load python modules.
import pylab
from scipy.optimize import curve_fit

# set interactive session or the canvas will freeze the execution
pylab.ion()

# Load the data (produced by plasduino/pendulumview)
t, x, x1 = pylab.loadtxt('data/analog_pendulum.txt', unpack = True)

#
# First of all let's take a look at the data
#
pylab.figure(1)
pylab.plot(t, x) 
pylab.title('Pendulum')
pylab.xlabel('Time [s]') 
pylab.ylabel('Position [a.u.]')
pylab.grid()

#
# Select a time interval and fit
#

# Time interval:
t_min = 100
t_max = 150

# Prepare a dataset for fitting:
# the easy way is to extract a subset of time-position data 
# that corresponds to the selected interval
t1 = t[t>t_min];  st = t1[t1<t_max];  
x1 = x[t>t_min];  sx = x1[t1<t_max]; 
#pylab.plot(st, sx) # plot this interval - for debug 

# Define fit function: a dumped harmonic oscillator
def f(x, A, omega, tau, p, c):
    return A*pylab.exp(-x/tau)*pylab.cos(omega*x +p) + c

# Set function initialization parameters
# Notice that these params are very important for algorithm convergence
initParams =pylab.array([500., 4.4, 70., 0., 480.])

# so let's take a look at function before the fit
# (uncomment the following lines)
#pylab.figure(2)
#pylab.plot(st, sx) 
#pylab.plot(st, f(st, initParams[0], initParams[1], initParams[2], 
#                 initParams[3], initParams[4]))
#pylab.title('Pendulum')
#pylab.xlabel('Time [s]') 
#pylab.ylabel('Position [a.u.]')
#pylab.grid()

# Fit and extract relevant parameters
popt, pcov = curve_fit(f, st, sx, initParams)
A_fit,   omega_fit,  tau_fit,  p_fit,  c_fit = popt
dA_fit, domega_fit, dtau_fit, dp_fit, dc_fit = pylab.sqrt(pcov.diagonal())
print('Omega = %f +- %f' % (omega_fit, domega_fit))
print('Tau   = %f +- %f' % (tau_fit,   dtau_fit))

# plot fit results and save figure
pylab.plot(st, f(st, popt[0], popt[1], popt[2], popt[3], popt[4]))

pylab.figure(3)
pylab.rc('font', size = 18)
pylab.title('Pendulum', y = 1.02)
pylab.xlabel('Time [s]') 
pylab.ylabel('Position [a.u.]')
pylab.grid(color = 'gray')
pylab.plot(st, sx, '+') 
pylab.plot(st, f(st, popt[0], popt[1], popt[2], 
                 popt[3], popt[4]))

pylab.savefig('analog_pendulum_fit.png')
