# This is a script make a histogram of a set of data and
# plot the analytic PDF over it
# matplotlib is used to plot and scipy for numerical integration
# 

# Load python modules.
import pylab
import numpy
from scipy import integrate

# set interactive session or the canvas will freeze the execution
pylab.ion()

# data: a pendulum period, each value is the average of 10 measurements
x = numpy.array([ 20.441, 20.463, 20.451, 20.321, 20.534, 20.450, 20.461, 
                  20.472, 20.462, 20.482, 20.479, 20.458, 20.393, 19.948, 
                  20.404, 20.554, 20.653, 20.519, 20.491, 20.567, 20.540, 
                  20.094, 20.469, 20.484, 20.206, 20.395, 20.409, 20.446, 
                  20.462, 20.503, 20.485, 20.575, 20.673, 20.448, 20.589, 
                  20.587, 20.577, 20.565, 20.522, 20.556, 20.577, 20.557, 
                  20.406, 20.417, 20.499, 20.471, 20.476, 20.525, 20.386, 
                  20.461, 20.490, 20.522, 20.594, 20.528, 20.419, 20.324, 
                  20.393, 20.479, 20.538, 20.469, 20.443, 20.533, 20.494, 
                  20.545, 20.392, 20.405, 20.395, 20.387, 20.582, 20.613], 
                'd')

# print some diagnostic info:
n = len(x)
x_min = x.min()
x_max = x.max()
print('Dataset: %d entries from %.3f to %.3f' % (n, x_min, x_max))
x_ave = numpy.sum(x)/n
x_rms = numpy.sqrt(numpy.sum((x-x_ave)**2)/(n-1))
print('Average = %.2f  -- RMS = %.2f' % (x_ave, x_rms))

# select number of bins 
nBins = 13

# the histogram of the data - let's use pylab histogram in its simplest 
# form and let it group data in nBins intervals from x.min() to x.max()
# http://matplotlib.org/1.2.1/api/pyplot_api.html#matplotlib.pyplot.hist
Oi, bins, patches = pylab.hist(x, nBins)
pylab.title('Period distribution')
pylab.xlabel('Period [s]') 
pylab.ylabel('Entries/bin')

# define gaussian
def g(x, N, m, s):
    return (N/(s*numpy.sqrt(2*numpy.pi)))*numpy.exp(-0.5*((x-m)/s)**2)

# plot expected gaussian function on top of the histogram
binWidth   = (bins[-1] -bins[0])/13
xg  = numpy.array([bins[0] + i*binWidth/10 for i in range(10*(nBins+1))]) 
pylab.plot(xg, g(xg, n*binWidth, x_ave, x_rms))

# loop over bins and eval Ei via numerical integration
# use scipy.integrate.quad routine, docs in
# http://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.quad.html
Ei = []
print ("binId\tbinMin\tbinMax\tOi\tEi")
for i in range(nBins):
    if i == 0:
            ei, d = integrate.quad(g, -numpy.inf, bins[1], \
                                       args=(n,x_ave, x_rms))
    elif i == nBins-1:
            ei, d = integrate.quad(g, bins[i], numpy.inf, \
                                       args=(n,x_ave, x_rms))
    else:
        ei, d = integrate.quad(g, bins[i], bins[i+1], \
                                   args=(n,x_ave, x_rms))
    Ei.append(ei)
    print ("%d\t%.2f\t%.2f\t%d\t%.3f" % \
               (i, bins[i], bins[i+1], Oi[i], ei))

Ei = numpy.array(Ei, 'd')
print("Sum of Ei = %.2f" % Ei.sum())

# evaluate Chi2 - grouping together bins in the tails
fBin = 6
lBin = nBins-1
Chi2 = ((Oi[:fBin].sum() - Ei[:fBin].sum())**2)/Ei[:fBin].sum() +\
    (((Oi[fBin:lBin] - Ei[fBin:lBin])**2)/Ei[fBin:lBin]).sum() +\
    ((Oi[lBin:].sum() - Ei[lBin:].sum())**2)/Ei[lBin:].sum()
print("Chi2 test grouping together the first %d and the last %d bins" %\
          (fBin-1, nBins-lBin))
print("Chi2  = %.2f; NDF = %d" % (Chi2, len(Oi[fBin:lBin])+2))
